﻿#region License Information
/* HeuristicLab
 * Copyright (C) Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HEAL.Attic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;

namespace HeuristicLab.Problems.DataAnalysis {

  [Item("Transformation", "Represents the base class for a transformation.")]
  [StorableType("46B380CC-6CDE-40FC-9BB4-E3FF82BA35EA")]
  public abstract class Transformation : ParameterizedNamedItem, ITransformation {
    protected const string ColumnParameterName = "Column";
    #region parameter properties
    public IConstrainedValueParameter<StringValue> ColumnParameter {
      get { return (IConstrainedValueParameter<StringValue>)Parameters[ColumnParameterName]; }
    }
    #endregion

    #region properties
    public abstract string ShortName { get; }

    public string Column {
      get { return ColumnParameter.Value.Value; }
      set {
        if (value == null) throw new ArgumentNullException("column", "The provided value for the column is null.");
        if (value == Column) return;

        var matchingParameterValue = ColumnParameter.ValidValues.FirstOrDefault(v => v.Value == value);
        ColumnParameter.Value = matchingParameterValue ?? throw new ArgumentException("The provided value is not valid as the column.", "column");
      }
    }
    #endregion

    [StorableConstructor]
    protected Transformation(StorableConstructorFlag _) : base(_) { }
    protected Transformation(Transformation original, Cloner cloner) : base(original, cloner) { }
    protected Transformation(IEnumerable<string> allowedColumns) {
      var allowed = new ItemSet<StringValue>(allowedColumns.Select(e => new StringValue(e)));
      Parameters.Add(new ConstrainedValueParameter<StringValue>(ColumnParameterName, "Column used for the Transformation", allowed, allowed.First()));
    }


    //TODO: Fill XML comments
    /// <summary>
    /// Creates transformation for the provided input variables. 
    /// </summary>
    /// <param name="transformation"></param>
    /// <param name="dataset"></param>
    /// <param name="rows"></param>
    /// <param name="allowedInputVariables"></param>
    /// <returns></returns>
    public static ITransformation<double>[] CreateTransformations(ITransformation<double> transformation, IDataset dataset, IEnumerable<int> rows, IEnumerable<string> allowedInputVariables) {
      var trans = new ITransformation<double>[allowedInputVariables.Count()];
      int i = 0;
      foreach (var variable in allowedInputVariables) {
        var trans_clone = (ITransformation<double>)transformation.Clone();
        trans_clone.Column = variable;
        trans_clone.ConfigureParameters(dataset.GetDoubleValues(variable, rows));
        trans[i] = trans_clone;
        i++;
      }
      return trans;
    }

  }

  [Item("Transformation", "Represents the base class for a transformation.")]
  [StorableType("F244DB89-72EA-4B41-89FA-C2E14F30AD29")]
  public abstract class Transformation<T> : Transformation, ITransformation<T> {

    [StorableConstructor]
    protected Transformation(StorableConstructorFlag _) : base(_) { }
    protected Transformation(Transformation<T> original, Cloner cloner) : base(original, cloner) { }
    protected Transformation(IEnumerable<string> allowedColumns) : base(allowedColumns) { }

    public virtual void ConfigureParameters(IEnumerable<T> data) {
      // override in transformations with parameters
    }

    public abstract IEnumerable<T> Apply(IEnumerable<T> data);
    public IEnumerable<T> ConfigureAndApply(IEnumerable<T> data) {
      ConfigureParameters(data);
      return Apply(data);
    }

    public abstract bool Check(IEnumerable<T> data, out string errorMsg);


  }
}
